#ifndef DEBTAGS_TAGDB_H
#define DEBTAGS_TAGDB_H

/*
 * System tag database
 *
 * Copyright (C) 2003  Enrico Zini <enrico@debian.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#pragma interface

#include <tagcoll/Exception.h>
#include <tagcoll/ParserBase.h>
#include <tagcoll/TagcollConsumer.h>
#include <tagcoll/InputMerger.h>
#include <tagcoll/Tag.h>
#include <debtags/Package.h>

namespace Tagcoll
{
class TagSet;
};

namespace Debtags
{
using namespace Tagcoll;

class Package;

/**
 * Access the local Debtags tag database.
 * TadDB provides read access to the local Debtags tag database via its various
 * output* methods.
 */
class TagDB : public InputMerger<Package, Tag>
{
public:
	/** Create a new TagDB
	 * @param editable Specifies if recording of modifications should be enabled.
	 * If editable is true, then the local state directory will be created when
	 * the object is instantiated
	 */
	TagDB(bool editable = false) throw (SystemException, ConsistencyCheckException);
	~TagDB() throw ();

	/// Get all the tags mentioned in the database
	TagSet getTags() const throw ();


	/** Load the database from disk
	 * if facet_only is set to true, then all tags without facets will be
	 * stripped on loading; if facet_only is set to false, then for all tags
	 * with a facet the loader will add a new tag with only the facet name
	 */
	void load(bool facet_only = true) throw (FileException, ParserException);


	/// Save in the state storage directory a patch that can be used to turn
	/// the system database into the collection given
	void savePatch()
		throw (FileException, ParserException, SystemException);

	/// Save in the state storage directory a patch to turn the system database
	/// into the collection given
	void savePatch(const InputMerger<Package, Tag>& collection)
		throw (FileException, ParserException, SystemException);

	/// Send to the central archive a patch that can be used to turn
	/// the system database into the collection given
	void sendPatch()
		throw (FileException, ParserException, SystemException, ConsistencyCheckException);

	/// Send to the central archive a patch to turn the system database
	/// into the collection given
	void sendPatch(const InputMerger<Package, Tag>& collection)
		throw (FileException, ParserException, SystemException, ConsistencyCheckException);
	

	/// Return the full path of the state storage directory (usually ~/.debtags)
	static const std::string& statedir() throw ();

	/// Returns the path to the current tag database (usually /var/lib/debtags/package-tags)
	static const std::string& path_tagdb() throw ();
	
	/// Check if the tag database has been created (i.e. if something
	/// equivalend to debtags update has been run)
	static bool hasTagDatabase() throw ();


	/** Output the database in the given file to a TagcollConsumer
	 * @note The collection is sent to 'cons' without merging repeated items
	 */
	static void outputFile(const std::string& name, TagcollConsumer<std::string, std::string>& cons, bool facet_only = true)
		throw (FileException, ParserException);

	/** Output the database in the given file to a TagcollConsumer
	 * @note The collection is sent to 'cons' without merging repeated items
	 */
	static void outputFile(const std::string& name, TagcollConsumer<std::string, Tag>& cons, bool facet_only = true)
		throw (FileException, ParserException);

	/** Output the database in the given file to a TagcollConsumer
	 * @note The collection is sent to 'cons' without merging repeated items
	 */
	static void outputFile(const std::string& name, TagcollConsumer<Package, Tag>& cons, bool facet_only = true)
		throw (FileException, ParserException);


	/** Output the current Debian tags database to a TagcollConsumer
	 * @note The collection is sent to 'cons' without merging repeated items
	 */
	static void outputSystem(TagcollConsumer<std::string, std::string>& cons, bool facet_only = true)
		throw (FileException, ParserException);

	/** Output the current Debian tags database to a TagcollConsumer
	 * @note The collection is sent to 'cons' without merging repeated items
	 */
	static void outputSystem(TagcollConsumer<std::string, Tag>& cons, bool facet_only = true)
		throw (FileException, ParserException);

	/** Output the current Debian tags database to a TagcollConsumer
	 * @note The collection is sent to 'cons' without merging repeated items
	 */
	static void outputSystem(TagcollConsumer<Package, Tag>& cons, bool facet_only = true)
		throw (FileException, ParserException);


	/// Output the current Debian tags database, patched with local patch, to a TagcollConsumer
	static void outputPatched(TagcollConsumer<std::string, std::string>& cons, bool facet_only = true)
		throw (FileException, ParserException);

	/// Output the current Debian tags database, patched with local patch, to a TagcollConsumer
	static void outputPatched(TagcollConsumer<std::string, Tag>& cons, bool facet_only = true)
		throw (FileException, ParserException);

	/// Output the current Debian tags database, patched with local patch, to a TagcollConsumer
	static void outputPatched(TagcollConsumer<Package, Tag>& cons, bool facet_only = true)
		throw (FileException, ParserException);
};

};

// vim:set ts=4 sw=4:
#endif
